package cn.iocoder.yudao.module.shipment.dal.mysql;

import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;

import cn.iocoder.yudao.module.shipment.dal.dataobject.MissNumberDO;

import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxEntity;
import cn.iocoder.yudao.module.shipment.enums.BoxAirStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.BoxStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.TransportTypeEnum;
import cn.iocoder.yudao.module.shipment.utils.NumberUtils;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.annotations.Mapper;
import cn.iocoder.yudao.module.shipment.vo.box.*;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

/**
 * 出货 Mapper
 *
 * @author lalala
 */
@Mapper
public interface BoxMapper extends AbstractMapper<BoxDO> {


    default PageResult<BoxBackVO> selectPage1(PageVO page, Object object) {
        if (object instanceof BoxQueryVO) {
            BoxQueryVO vo = (BoxQueryVO) object;
            if (StringUtils.isEmpty(vo.getTransportType())) {
                vo.setTransportType(null);
            }
            List<Long> warehouseIdList = vo.getWarehouseIdList();

            String lastSql = null;
            if (vo.getLadingBillStatus() != null) {
                if (vo.getLadingBillStatus() == 0) {
                    lastSql = " having ladingCount=0";
                } else if (vo.getLadingBillStatus() == 1) {
                    lastSql = " having ladingCount <> orderCount and ladingCount > 0";
                } else if (vo.getLadingBillStatus() == 2) {
                    lastSql = " having ladingCount = orderCount and ladingCount > 0";
                }
            }

            IPage<BoxEntity> mpPage = MyBatisUtils.buildPage(page);

            LambdaQueryWrapper<BoxDO> queryWrapper = new QueryWrapper<BoxDO>()
                    .select(" box.*, (SELECT count(DISTINCT order_id) FROM ecw_box_preload_goods preload WHERE preload.shipment_id = box.id AND preload.deleted = 0 AND preload.is_remove = 0 ) AS orderCount,\n" +
                            "\t( SELECT count(*) FROM ecw_make_bill_of_lading lading WHERE lading.shipment_id = box.id AND lading.deleted = 0 AND lading.`status` = 2 ) AS ladingCount ")
                    .lambda()
                    .eq(vo.getId() != null, BoxDO::getId, vo.getId())
                    .like(vo.getSelfNo() != null, BoxDO::getSelfNo, vo.getSelfNo())
                    .like(vo.getCubNo() != null, BoxDO::getCubNo, vo.getCubNo())
                    .eq(vo.getCabinetId() != null, BoxDO::getCabinetId, vo.getCabinetId())
                    .eq(vo.getStartWarehouseId() != null, BoxDO::getStartWarehouseId, vo.getStartWarehouseId())
                    .eq(vo.getBoxStatus() != null, BoxDO::getShipmentStatus, vo.getBoxStatus())
                    .eq(vo.getDestWarehouseId() != null, BoxDO::getDestWarehouseId, vo.getDestWarehouseId())
                    .eq(vo.getTransportType() != null, BoxDO::getTransportType, vo.getTransportType())
                    .in(CollectionUtil.isNotEmpty(vo.getTransportTypeList()), BoxDO::getTransportType, vo.getTransportTypeList())
                    .eq(vo.getShippingChannelId() != null, BoxDO::getShippingChannelId, vo.getShippingChannelId())
                    .and(CollectionUtil.isNotEmpty(warehouseIdList),
                            i -> i.in(BoxDO::getStartWarehouseId, warehouseIdList)
                                    .or()
                                    .in(BoxDO::getDestWarehouseId, warehouseIdList)
                    )
                    .and(NumberUtils.hasCon(vo), wrapper -> {
                                NumberUtils.buildWrapper(BoxDO::getYzDate, vo.getBeginYzDate(), vo.getEndYzDate(), wrapper);
                                wrapper.or(NumberUtils.hasCon(vo.getBeginZgDate(), vo.getEndZgDate()), zgWrapper -> NumberUtils.buildWrapper(BoxDO::getZgDate, vo.getBeginZgDate(), vo.getEndZgDate(), zgWrapper));
                                wrapper.or(NumberUtils.hasCon(vo.getBeginQyDate(), vo.getEndQyDate()), qyWrapper -> NumberUtils.buildWrapper(BoxDO::getQyDate, vo.getBeginQyDate(), vo.getEndQyDate(), qyWrapper));
                                wrapper.or(NumberUtils.hasCon(vo.getBeginDgDate(), vo.getEndDgDate()), dgWrapper -> NumberUtils.buildWrapper(BoxDO::getDgDate, vo.getBeginDgDate(), vo.getEndDgDate(), dgWrapper));
                                wrapper.or(NumberUtils.hasCon(vo.getBeginQgDate(), vo.getEndQgDate()), qgWrapper -> NumberUtils.buildWrapper(BoxDO::getQgDate, vo.getBeginQgDate(), vo.getEndQgDate(), qgWrapper));
                                wrapper.or(NumberUtils.hasCon(vo.getBeginThDate(), vo.getEndThDate()), xgWrapper -> NumberUtils.buildWrapper(BoxDO::getThDate, vo.getBeginThDate(), vo.getEndThDate(), xgWrapper));
                                wrapper.or(NumberUtils.hasCon(vo.getBeginCkDate(), vo.getEndCkDate()), ckWrapper -> NumberUtils.buildWrapper(BoxDO::getCkDate, vo.getBeginCkDate(), vo.getEndCkDate(), ckWrapper));
                            }
                    )
                    .ge(vo.getLadingBillStatus() != null && vo.getLadingBillStatus() > 0, BoxDO::getShipmentStatus, BoxStatusEnum.SEAL_EXAMINE_SUCCESS.getStatus())
                    .last(StringUtils.isNotBlank(lastSql), lastSql + " ORDER BY id DESC")
//                    .orderByDesc(BoxDO::getId));
                    .last(StringUtils.isBlank(lastSql), "ORDER BY id DESC");

//            String whereSql = queryWrapper.getCustomSqlSegment();
//            String ss = queryWrapper.getTargetSql();
//            String sss = queryWrapper.getSqlSegment();
//            String selectSql = queryWrapper.getSqlSelect();

//            String sql = "select " + selectSql + " from ecw_box " + whereSql;
            IPage<Map<String, Object>> list = boxEntityList(queryWrapper, mpPage);

            IPage<BoxBackVO> convertedPage = new Page<>();

            // 设置IPage的基本属性
            convertedPage.setCurrent(list.getCurrent());
            convertedPage.setSize(list.getSize());
            convertedPage.setTotal(list.getTotal());
            convertedPage.setPages(list.getPages());

            List<BoxBackVO> records = list.getRecords().stream().map(recordMap -> {
                BoxBackVO boxEntity = new BoxBackVO();
                boxEntity.setId((Long) recordMap.get("id"));
                boxEntity.setSelfNo((String) recordMap.get("self_no"));
                boxEntity.setCubNo((String) recordMap.get("cub_no"));
                boxEntity.setCabinetId((Long) recordMap.get("cabinet_id"));
                boxEntity.setStartWarehouseId((Long) recordMap.get("start_warehouse_id"));
                boxEntity.setDestWarehouseId((Long) recordMap.get("dest_warehouse_id"));
                boxEntity.setTransportType((String) recordMap.get("transport_type"));
                boxEntity.setShippingChannelId((Long) recordMap.get("shipping_channel_id"));
                boxEntity.setBkStatus((Integer) recordMap.get("bk_status"));
                boxEntity.setPrStatus((Integer) recordMap.get("pr_status"));
                boxEntity.setTyStatus((Integer) recordMap.get("ty_status"));
                boxEntity.setTlStatus((Integer) recordMap.get("tl_status"));
                boxEntity.setTlContainerStatus((Integer) recordMap.get("tl_container_status"));
                boxEntity.setLdStatus((Integer) recordMap.get("ld_status"));
                boxEntity.setDcStatus((Integer) recordMap.get("dc_status"));
                boxEntity.setDcCustomsStatus((Integer) recordMap.get("dc_customs_status"));
                boxEntity.setDcCheckStatus((Integer) recordMap.get("dc_check_status"));
                boxEntity.setCheckExamineStatus((Integer) recordMap.get("check_examine_status"));
                boxEntity.setCheckDealStatus((Integer) recordMap.get("check_deal_status"));
                boxEntity.setSaStatus((Integer) recordMap.get("sa_status"));
                boxEntity.setBlStatus((Integer) recordMap.get("bl_status"));
                boxEntity.setBgStatus((Integer) recordMap.get("bg_status"));
                boxEntity.setDtStatus((Integer) recordMap.get("dt_status"));
                boxEntity.setCpStatus((Integer) recordMap.get("cp_status"));
                boxEntity.setCdStatus((Integer) recordMap.get("cd_status"));
                boxEntity.setApStatus((Integer) recordMap.get("ap_status"));
                boxEntity.setClStatus((Integer) recordMap.get("cl_status"));
                boxEntity.setStkStatus((Integer) recordMap.get("stk_status"));
                boxEntity.setSapStatus((Integer) recordMap.get("sap_status"));
                boxEntity.setUlStatus((Integer) recordMap.get("ul_status"));
                boxEntity.setSlStatus((Integer) recordMap.get("sl_status"));
                boxEntity.setShipmentStatus((Integer) recordMap.get("shipment_status"));
                LocalDateTime localDateTime = (LocalDateTime) recordMap.get("create_time");
                Timestamp timestamp = Timestamp.valueOf(localDateTime);
                Date date = new Date(timestamp.getTime());
                boxEntity.setCreateTime(date);
                boxEntity.setSortingStatus((Integer) recordMap.get("sorting_status"));
                boxEntity.setCheckoutStatus((Integer) recordMap.get("checkout_status"));
                boxEntity.setShipmentStatusAir((Integer) recordMap.get("shipment_status_air"));


                Long orderCount = (Long) recordMap.get("orderCount");
                Long ladingCount = (Long) recordMap.get("ladingCount");
                boxEntity.setLadingBillStatus(0);
                if (orderCount != 0) {
                    if (ladingCount > 0) {
                        if (ladingCount >= orderCount) {
                            boxEntity.setLadingBillStatus(2);
                        } else {
                            boxEntity.setLadingBillStatus(1);
                        }
                    }
                }
                return boxEntity;
            }).collect(Collectors.toList());

            convertedPage.setRecords(records);

            return PageResult.of(convertedPage);
        }
        return null;
    }

    default PageResult<BoxBackVO> selectPage2(PageVO page, Object object) {
        if (object instanceof BoxQueryVO) {
            BoxQueryVO vo = (BoxQueryVO) object;
            if (StringUtils.isEmpty(vo.getTransportType())) {
                vo.setTransportType(null);
            }
            List<Long> warehouseIdList = vo.getWarehouseIdList();

            String lastSql = null;
            if (vo.getLadingBillStatus() != null) {
                if (vo.getLadingBillStatus() == 0) {
                    lastSql = " and ladingCount=0";
                } else if (vo.getLadingBillStatus() == 1) {
                    lastSql = " and ladingCount <> orderCount and ladingCount > 0";
                } else if (vo.getLadingBillStatus() == 2) {
                    lastSql = " and ladingCount = orderCount and ladingCount > 0";
                }
            }

            IPage<BoxEntity> mpPage = MyBatisUtils.buildPage(page);

            boolean air = vo.getTransportType() != null
                    && vo.getTransportType().equals(TransportTypeEnum.AIR.getType());

            String bookSeaSql = buildBookSeaSql(vo);
            String customsSql = buildCustomsSql(vo);
            String shippingSql = buildShippingSql(vo);
            String clearanceDocSql = buildClearanceDocSql(vo);
            String clearanceSql = buildClearanceSql(vo);
            String buildBookAirSql = buildBookAirString(vo);
            String buildAgentSql = buildAgentSql(vo);

            LambdaQueryWrapper<BoxDO> queryWrapper = new LambdaQueryWrapper<BoxDO>()
                    .eq(vo.getId() != null, BoxDO::getId, vo.getId())
                    .eq(BoxDO::getDeleted, 0)
                    .like(vo.getSelfNo() != null, BoxDO::getSelfNo, vo.getSelfNo())
                    .like(vo.getCubNo() != null, BoxDO::getCubNo, vo.getCubNo())
                    .eq(vo.getCabinetId() != null, BoxDO::getCabinetId, vo.getCabinetId())
                    .eq(vo.getStartWarehouseId() != null, BoxDO::getStartWarehouseId, vo.getStartWarehouseId())
                    .in(CollectionUtil.isNotEmpty(vo.getStartWarehouseIdList()),BoxDO::getStartWarehouseId,vo.getStartWarehouseIdList())
                    .in(CollectionUtil.isNotEmpty(vo.getDestWarehouseIdList()),BoxDO::getDestWarehouseId,vo.getDestWarehouseIdList())
                    .in(CollectionUtil.isNotEmpty(vo.getBoxStatusList()),BoxDO::getStatus,vo.getBoxStatusList())
                    .in(CollectionUtil.isNotEmpty(vo.getDestinationClearanceList()),BoxDO::getDestinationClearance,vo.getDestinationClearanceList())
                    .in(CollectionUtil.isNotEmpty(vo.getDeliveryTypeList()), BoxDO::getDeliveryType, vo.getDeliveryTypeList())
                    .in(CollectionUtil.isNotEmpty(vo.getShippingChannelIdList()), BoxDO::getShippingChannelId, vo.getShippingChannelIdList())
//                    .apply(CollectionUtil.isNotEmpty(vo.getBigPortIdList()),"exists (select 1 from ecw_box_book_sea a where a.shipment_id={0} and a.deleted=0 and a.big_port_id in ({1}))",vo.getId(),vo.getBigPortIdList())
//                    .apply(CollectionUtil.isNotEmpty(vo.getDestPortIdList()),"exists (select 1 from ecw_box_book_sea a where a.shipment_id={0} and a.deleted=0 and a.dest_port_id in ({1}))",vo.getId(),vo.getDestPortIdList())
//                    .apply(CollectionUtil.isNotEmpty(vo.getBargePortIdList()),"exists (select 1 from ecw_box_book_sea a where a.shipment_id={0} and a.deleted=0 and a.barge_port_id in ({1}))",vo.getId(),vo.getBargePortIdList())
//                    .apply(CollectionUtil.isNotEmpty(vo.getSpaceCompanyIdList()),"exists (select 1 from ecw_box_book_sea a where a.shipment_id={0} and a.deleted=0 and a.space_company_id in ({1}))",vo.getId(),vo.getSpaceCompanyIdList())
                    .apply(StringUtils.isNotBlank(bookSeaSql),bookSeaSql,vo.getId())
                    .apply(StringUtils.isNotBlank(customsSql),customsSql,vo.getId())
                    .apply(StringUtils.isNotBlank(shippingSql),shippingSql,vo.getId())
                    .apply(StringUtils.isNotBlank(clearanceDocSql),clearanceDocSql,vo.getId())
                    .apply(StringUtils.isNotBlank(clearanceSql),clearanceSql,vo.getId())
                    .apply(StringUtils.isNotBlank(buildBookAirSql),buildBookAirSql,vo.getId())
                    .apply(StringUtils.isNotBlank(buildAgentSql),buildAgentSql,vo.getId())
                    .eq(vo.getBoxStatus() != null, BoxDO::getShipmentStatus, vo.getBoxStatus())
                    .eq(vo.getShipmentStatusAir() != null, BoxDO::getShipmentStatusAir, vo.getShipmentStatusAir())
                    .eq(vo.getDestWarehouseId() != null, BoxDO::getDestWarehouseId, vo.getDestWarehouseId())
                    .eq(vo.getTransportType() != null, BoxDO::getTransportType, vo.getTransportType())
                    .eq(vo.getDestinationClearance() != null, BoxDO::getDestinationClearance, vo.getDestinationClearance())
                    .eq(vo.getDeliveryType() != null, BoxDO::getDeliveryType, vo.getDeliveryType())
                    .in(CollectionUtil.isNotEmpty(vo.getTransportTypeList()), BoxDO::getTransportType, vo.getTransportTypeList())
                    .eq(vo.getShippingChannelId() != null, BoxDO::getShippingChannelId, vo.getShippingChannelId())
                    .and(CollectionUtil.isNotEmpty(warehouseIdList),
                            i -> i.in(BoxDO::getStartWarehouseId, warehouseIdList)
                                    .or()
                                    .in(BoxDO::getDestWarehouseId, warehouseIdList)
                    )
                    .and(NumberUtils.hasCon(vo), wrapper -> {
                                NumberUtils.buildWrapper(BoxDO::getYzDate, vo.getBeginYzDate(), vo.getEndYzDate(), wrapper);
                                wrapper.or(NumberUtils.hasCon(vo.getBeginZgDate(), vo.getEndZgDate()), zgWrapper -> NumberUtils.buildWrapper(BoxDO::getZgDate, vo.getBeginZgDate(), vo.getEndZgDate(), zgWrapper));
                                wrapper.or(NumberUtils.hasCon(vo.getBeginQyDate(), vo.getEndQyDate()), qyWrapper -> NumberUtils.buildWrapper(BoxDO::getQyDate, vo.getBeginQyDate(), vo.getEndQyDate(), qyWrapper));
                                wrapper.or(NumberUtils.hasCon(vo.getBeginDgDate(), vo.getEndDgDate()), dgWrapper -> NumberUtils.buildWrapper(BoxDO::getDgDate, vo.getBeginDgDate(), vo.getEndDgDate(), dgWrapper));
                                wrapper.or(NumberUtils.hasCon(vo.getBeginQgDate(), vo.getEndQgDate()), qgWrapper -> NumberUtils.buildWrapper(BoxDO::getQgDate, vo.getBeginQgDate(), vo.getEndQgDate(), qgWrapper));
                                wrapper.or(NumberUtils.hasCon(vo.getBeginCkDate(), vo.getEndCkDate()), ckWrapper -> NumberUtils.buildWrapper(BoxDO::getCkDate, vo.getBeginCkDate(), vo.getEndCkDate(), ckWrapper));
                                wrapper.or(NumberUtils.hasCon(vo.getBeginThDate(), vo.getEndThDate()), xgWrapper -> NumberUtils.buildWrapper(BoxDO::getThDate, vo.getBeginThDate(), vo.getEndThDate(), xgWrapper));
                            }
                    )
                    .ge(vo.getLadingBillStatus() != null && vo.getLadingBillStatus() > 0 && !air, BoxDO::getShipmentStatus, BoxStatusEnum.SEAL_EXAMINE_SUCCESS.getStatus())
                    .ge(vo.getLadingBillStatus() != null && vo.getLadingBillStatus() > 0 && air, BoxDO::getShipmentStatusAir, BoxAirStatusEnum.AIR_SHIPPED.getStatus())
                    .last(StringUtils.isNotBlank(lastSql), lastSql + " ORDER BY id DESC")
//                    .orderByDesc(BoxDO::getId));
                    .last(StringUtils.isBlank(lastSql), "ORDER BY id DESC");

//            String whereSql = queryWrapper.getCustomSqlSegment();
//            String ss = queryWrapper.getTargetSql();
//            String sss = queryWrapper.getSqlSegment();
//            String selectSql = queryWrapper.getSqlSelect();

//            String sql = "select " + selectSql + " from ecw_box " + whereSql;
            IPage<Map<String, Object>> list = boxEntityList1(queryWrapper, mpPage);

            IPage<BoxBackVO> convertedPage = new Page<>();

            // 设置IPage的基本属性
            convertedPage.setCurrent(list.getCurrent());
            convertedPage.setSize(list.getSize());
            convertedPage.setTotal(list.getTotal());
            convertedPage.setPages(list.getPages());

            List<BoxBackVO> records = list.getRecords().stream().map(recordMap -> {
                BoxBackVO boxEntity = new BoxBackVO();
                boxEntity.setId((Long) recordMap.get("id"));
                boxEntity.setSelfNo((String) recordMap.get("self_no"));
                boxEntity.setCubNo((String) recordMap.get("cub_no"));
                boxEntity.setCabinetId((Long) recordMap.get("cabinet_id"));
                boxEntity.setStartWarehouseId((Long) recordMap.get("start_warehouse_id"));
                boxEntity.setDestWarehouseId((Long) recordMap.get("dest_warehouse_id"));
                boxEntity.setTransportType((String) recordMap.get("transport_type"));
                boxEntity.setShippingChannelId((Long) recordMap.get("shipping_channel_id"));
                boxEntity.setBkStatus((Integer) recordMap.get("bk_status"));
                boxEntity.setPrStatus((Integer) recordMap.get("pr_status"));
                boxEntity.setTyStatus((Integer) recordMap.get("ty_status"));
                boxEntity.setTlStatus((Integer) recordMap.get("tl_status"));
                boxEntity.setTlContainerStatus((Integer) recordMap.get("tl_container_status"));
                boxEntity.setLdStatus((Integer) recordMap.get("ld_status"));
                boxEntity.setDcStatus((Integer) recordMap.get("dc_status"));
                boxEntity.setDcCustomsStatus((Integer) recordMap.get("dc_customs_status"));
                boxEntity.setDcCheckStatus((Integer) recordMap.get("dc_check_status"));
                boxEntity.setCheckExamineStatus((Integer) recordMap.get("check_examine_status"));
                boxEntity.setCheckDealStatus((Integer) recordMap.get("check_deal_status"));
                boxEntity.setSaStatus((Integer) recordMap.get("sa_status"));
                boxEntity.setBlStatus((Integer) recordMap.get("bl_status"));
                boxEntity.setBgStatus((Integer) recordMap.get("bg_status"));
                boxEntity.setDtStatus((Integer) recordMap.get("dt_status"));
                boxEntity.setCpStatus((Integer) recordMap.get("cp_status"));
                boxEntity.setCdStatus((Integer) recordMap.get("cd_status"));
                boxEntity.setApStatus((Integer) recordMap.get("ap_status"));
                boxEntity.setClStatus((Integer) recordMap.get("cl_status"));
                boxEntity.setStkStatus((Integer) recordMap.get("stk_status"));
                boxEntity.setSapStatus((Integer) recordMap.get("sap_status"));
                boxEntity.setUlStatus((Integer) recordMap.get("ul_status"));
                boxEntity.setSlStatus((Integer) recordMap.get("sl_status"));
                boxEntity.setShipmentStatus((Integer) recordMap.get("shipment_status"));
                LocalDateTime localDateTime = (LocalDateTime) recordMap.get("create_time");
                Timestamp timestamp = Timestamp.valueOf(localDateTime);
                Date date = new Date(timestamp.getTime());
                boxEntity.setCreateTime(date);
                boxEntity.setSortingStatus((Integer) recordMap.get("sorting_status"));
                boxEntity.setCheckoutStatus((Integer) recordMap.get("checkout_status"));
                boxEntity.setAirShipmentStatus((Integer) recordMap.get("air_shipment_status"));
                boxEntity.setMergePkgStatus((Integer) recordMap.get("merge_pkg_status"));
                boxEntity.setToWarehouseStatus((Integer) recordMap.get("to_warehouse_status"));
                boxEntity.setDestinationClearance((Integer) recordMap.get("destination_clearance"));
                boxEntity.setDeliveryType((Integer) recordMap.get("delivery_type"));
                boxEntity.setFlyStatus((Integer) recordMap.get("fly_status"));
                boxEntity.setShipmentStatusAir((Integer) recordMap.get("shipment_status_air"));
                boxEntity.setDestCountryId((Long) recordMap.get("dest_country_id"));

                Long orderCount = (Long) recordMap.get("orderCount");
                Long ladingCount = (Long) recordMap.get("ladingCount");
                boxEntity.setLadingBillStatus(0);
                if (orderCount != 0) {
                    if (ladingCount > 0) {
                        if (orderCount.equals(ladingCount)) {
                            boxEntity.setLadingBillStatus(2);
                        } else if (orderCount < ladingCount) {
                            // 添加的逻辑，提单有可能补单
                            boxEntity.setLadingBillStatus(2);
                        } else {
                            boxEntity.setLadingBillStatus(1);
                        }
                    }
                }
                return boxEntity;
            }).collect(Collectors.toList());

            convertedPage.setRecords(records);

            return PageResult.of(convertedPage);
        }
        return null;
    }

    //拼装海运订舱sql
    default String buildBookSeaSql(BoxQueryVO vo) {
        boolean flag = false;
        StringBuffer stringBuffer = new StringBuffer("exists (select 1 from ecw_box_book_sea a where a.shipment_id={0} and a.deleted=0 ");
        if (CollectionUtil.isNotEmpty(vo.getBigPortIdList())) {
            flag = true;
            String collect = vo.getBigPortIdList().stream().map(item -> item.toString()).collect(Collectors.joining(","));
            stringBuffer.append(" and a.big_port_id in (").append(collect).append(")");
        }
        if (CollectionUtil.isNotEmpty(vo.getDestPortIdList())) {
            flag = true;
            String collect = vo.getDestPortIdList().stream().map(item -> item.toString()).collect(Collectors.joining(","));
            stringBuffer.append(" and a.dest_port_id in (").append(collect).append(")");
        }
        if (CollectionUtil.isNotEmpty(vo.getBargePortIdList())) {
            flag = true;
            String collect = vo.getBargePortIdList().stream().map(item -> item.toString()).collect(Collectors.joining(","));
            stringBuffer.append(" and a.barge_port_id in (").append(collect).append(")");
        }
        if (CollectionUtil.isNotEmpty(vo.getSpaceCompanyIdList())) {
            flag = true;
            String collect = vo.getSpaceCompanyIdList().stream().map(item -> item.toString()).collect(Collectors.joining(","));
            stringBuffer.append(" and a.space_company_id in (").append(collect).append(")");
        }
        if (vo.getBargeBeginTime() !=null && vo.getBargeEndTime() != null) {
            flag = true;
            stringBuffer.append(" and a.barge_time between '").append(vo.getBargeBeginTime()).append("' and '").append(vo.getBargeEndTime()).append("'");
        }
        if (vo.getSailBeginTime() !=null && vo.getSailEndTime() != null) {
            flag = true;
            stringBuffer.append(" and a.sail_time between '").append(vo.getSailBeginTime()).append("' and '").append(vo.getSailEndTime()).append("'");
        }
        if (vo.getLadingBillBeginTime() !=null && vo.getLadingBillEndTime() != null) {
            flag = true;
            stringBuffer.append(" and a.lading_bill_time between '").append(vo.getLadingBillBeginTime()).append("' and '").append(vo.getLadingBillEndTime()).append("'");
        }
        if (vo.getCustomsBeginTime() !=null && vo.getCustomsEndTime() != null) {
            flag = true;
            stringBuffer.append(" and a.customs_time between '").append(vo.getCustomsBeginTime()).append("' and '").append(vo.getCustomsEndTime()).append("'");
        }
        if (vo.getCutOffBeginTime() !=null && vo.getCutOffEndTime() != null) {
            flag = true;
            stringBuffer.append(" and a.cut_off_time between '").append(vo.getCutOffBeginTime()).append("' and '").append(vo.getCutOffEndTime()).append("'");
        }
        if (flag) {
            stringBuffer.append(")");
            return stringBuffer.toString();
        }
        return null;
    }

    //拼装报关查询sql
    default String buildCustomsSql(BoxQueryVO vo) {
        boolean flag = false;
        StringBuffer stringBuffer = new StringBuffer("exists (select 1 from ecw_box_customs a where a.shipment_id={0} and a.deleted=0 ");
        if (CollectionUtil.isNotEmpty(vo.getDcCustomsTypeList())) {
            flag = true;
            String collect = vo.getDcCustomsTypeList().stream().map(item -> item.toString()).collect(Collectors.joining(","));
            stringBuffer.append(" and a.dc_customs_type in (").append(collect).append(")");
        }
        if (CollectionUtil.isNotEmpty(vo.getDcCompanyIdList())) {
            flag = true;
            String collect = vo.getDcCompanyIdList().stream().map(item -> item.toString()).collect(Collectors.joining(","));
            stringBuffer.append(" and a.dc_company_id in (").append(collect).append(")");
        }
        if (vo.getDcPassBeginTime() !=null && vo.getDcPassEndTime() != null) {
            flag = true;
            stringBuffer.append(" and a.dc_pass_time between '").append(vo.getDcPassBeginTime()).append("' and '").append(vo.getDcPassEndTime()).append("'");
        }
        if (flag) {
            stringBuffer.append(")");
            return stringBuffer.toString();
        }
        return null;
    }

    default String buildShippingSql(BoxQueryVO vo) {
        boolean flag = false;
        StringBuffer stringBuffer = new StringBuffer("exists (select 1 from ecw_box_shipping a where a.shipment_id={0} and a.deleted=0 ");
        if (vo.getDtRealShipBeginTime() !=null && vo.getDtRealShipEndTime() != null) {
            flag = true;
            stringBuffer.append(" and a.dt_real_ship_time between '").append(vo.getDtRealShipBeginTime()).append("' and '").append(vo.getDtRealShipEndTime()).append("'");
        }
        if (vo.getDtEstArrivalBeginTime() !=null && vo.getDtEstArrivalEndTime() != null) {
            flag = true;
            stringBuffer.append(" and a.dt_est_arrival_time between '").append(vo.getDtEstArrivalBeginTime()).append("' and '").append(vo.getDtEstArrivalEndTime()).append("'");
        }
        if (flag) {
            stringBuffer.append(")");
            return stringBuffer.toString();
        }
        return null;
    }

    default String buildClearanceDocSql(BoxQueryVO vo) {
        boolean flag = false;
        StringBuffer stringBuffer = new StringBuffer("exists (select 1 from ecw_box_clearance_doc a where a.shipment_id={0} and a.deleted=0 ");
        if (CollectionUtil.isNotEmpty(vo.getCdOutBillTypeList())) {
            flag = true;
            String collect = vo.getCdOutBillTypeList().stream().map(item -> item.toString()).collect(Collectors.joining(","));
            stringBuffer.append(" and a.cd_out_bill_type in (").append(collect).append(")");
        }
        if (vo.getCdAgentlistType() !=null && vo.getCdAgentlistType() >0) {
            flag = true;
            stringBuffer.append(" and a.cd_agentlist_type=").append(vo.getCdAgentlistType());
        }
        if (vo.getCdSoncapType() !=null && vo.getCdSoncapType() != null) {
            flag = true;
            stringBuffer.append(" and a.cd_soncap_type=").append(vo.getCdSoncapType());
        }
        if (flag) {
            stringBuffer.append(")");
            return stringBuffer.toString();
        }
        return null;
    }

    default String buildClearanceSql(BoxQueryVO vo) {
        boolean flag = false;
        StringBuffer stringBuffer = new StringBuffer("exists (select 1 from ecw_box_clearance a where a.shipment_id={0} and a.deleted=0 ");
        if (vo.getClEstBeginTime() !=null && vo.getClEstEndTime() != null) {
            flag = true;
            stringBuffer.append(" and a.cl_est_time between '").append(vo.getClEstBeginTime()).append("' and '").append(vo.getClEstEndTime()).append("'");
        }
        if (flag) {
            stringBuffer.append(")");
            return stringBuffer.toString();
        }
        return null;
    }

    default String buildBookAirString(BoxQueryVO vo) {
        boolean flag = false;
        StringBuffer stringBuffer = new StringBuffer("exists (select 1 from ecw_box_book_air a where a.shipment_id={0} and a.deleted=0 ");
        if (CollectionUtil.isNotEmpty(vo.getDeparturePortIdList())) {
            flag = true;
            String collect = vo.getDeparturePortIdList().stream().map(item -> item.toString()).collect(Collectors.joining(","));
            stringBuffer.append(" and a.departure_port_id in (").append(collect).append(")");
        }
        if (CollectionUtil.isNotEmpty(vo.getAirDestPortIdList())) {
            flag = true;
            String collect = vo.getAirDestPortIdList().stream().map(item -> item.toString()).collect(Collectors.joining(","));
            stringBuffer.append(" and a.dest_port_id in (").append(collect).append(")");
        }
        if (vo.getVoyage() !=null && vo.getVoyage() >0) {
            flag = true;
            stringBuffer.append(" and a.voyage=").append(vo.getVoyage());
        }
        if (CollectionUtil.isNotEmpty(vo.getShipperIdList())) {
            flag = true;
            String collect = vo.getShipperIdList().stream().map(item -> item.toString()).collect(Collectors.joining(","));
            stringBuffer.append(" and a.shipper_id in (").append(collect).append(")");
        }
        if (CollectionUtil.isNotEmpty(vo.getAirlineCompanyIdList())) {
            flag = true;
            String collect = vo.getAirlineCompanyIdList().stream().map(item -> item.toString()).collect(Collectors.joining(","));
            stringBuffer.append(" and a.airline_company_id in (").append(collect).append(")");
        }
        if (vo.getFlyBeginTime() !=null && vo.getFlyEndTime() != null) {
            flag = true;
            stringBuffer.append(" and a.fly_time between '").append(vo.getFlyBeginTime()).append("' and '").append(vo.getFlyEndTime()).append("'");
        }
        if (flag) {
            stringBuffer.append(")");
            return stringBuffer.toString();
        }
        return null;
    }

    default String buildAgentSql(BoxQueryVO vo) {
        boolean flag = false;
        StringBuffer stringBuffer = new StringBuffer("exists (select 1 from ecw_box_agent a where a.shipment_id={0} and a.deleted=0 ");
        if (CollectionUtil.isNotEmpty(vo.getAgentIdList())) {
            flag = true;
            String collect = vo.getAgentIdList().stream().map(item -> item.toString()).collect(Collectors.joining(","));
            stringBuffer.append(" and a.agent_id in (").append(collect).append(")");
        }
        if (flag) {
            stringBuffer.append(")");
            return stringBuffer.toString();
        }
        return null;
    }


    @Override
    default List<BoxDO> selectList(Object object) {
        if (object instanceof BoxQueryVO) {
            BoxQueryVO vo = (BoxQueryVO) object;
            return selectList(new LambdaQuery<BoxDO>()
                    .eqIfPresent(BoxDO::getId, vo.getId())
                    .eqIfPresent(BoxDO::getSelfNo, vo.getSelfNo())
                    .eqIfPresent(BoxDO::getCubNo, vo.getCubNo())
                    .eqIfPresent(BoxDO::getCabinetId, vo.getCabinetId())
                    .eqIfPresent(BoxDO::getStartWarehouseId, vo.getStartWarehouseId())
                    .eqIfPresent(BoxDO::getStatus, vo.getStatus())
                    .eqIfPresent(BoxDO::getDestWarehouseId, vo.getDestWarehouseId())
                    .eqIfPresent(BoxDO::getTransportType, vo.getTransportType())
                    .eqIfPresent(BoxDO::getShippingChannelId, vo.getShippingChannelId())
                    .betweenIfPresent(BoxDO::getYzDate, vo.getBeginYzDate(), vo.getEndYzDate())
                    .betweenIfPresent(BoxDO::getZgDate, vo.getBeginZgDate(), vo.getEndZgDate())
                    .betweenIfPresent(BoxDO::getQyDate, vo.getBeginQyDate(), vo.getEndQyDate())
                    .betweenIfPresent(BoxDO::getDgDate, vo.getBeginDgDate(), vo.getEndDgDate())
                    .betweenIfPresent(BoxDO::getQgDate, vo.getBeginQgDate(), vo.getEndQgDate())
                    .betweenIfPresent(BoxDO::getThDate, vo.getBeginThDate(), vo.getEndThDate())
                    .betweenIfPresent(BoxDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(BoxDO::getId));
        }

        return null;
    }

    @ResultType(BoxDO.class)
    @Select({
            "<script>",
            "SELECT",
            "box.*",
            "FROM",
            "ecw_box box,",
            "ecw_box_agent agent ",
            "WHERE",
            "box.id = agent.shipment_id",
            "AND box.deleted = 0",
            "AND agent.deleted = 0",
            "and box.pr_status = 25",
            "and box.ty_status = 2111",
            "AND box.transport_type in ",
            "<foreach item='id' index='index' collection='transportTypeList' open='(' separator=',' close=')'>",
            "#{id}",
            "</foreach>",
            "AND (box.start_warehouse_id in ",
            "<foreach item='id' index='index' collection='warehouseIdList' open='(' separator=',' close=')'>",
            "#{id}",
            "</foreach> or box.dest_warehouse_id in ",
            "<foreach item='id' index='index' collection='warehouseIdList' open='(' separator=',' close=')'>",
            "#{id}",
            "</foreach>)",
            "<when test = 'selfNo != null and selfNo != \"\" '>",
            "and box.self_no LIKE concat('%',#{selfNo},'%')",
            "</when>",
            "</script>"
    })
    IPage<BoxDO> tallyPage(@Param("mpPage") IPage<BoxDO> page,
                           @Param("warehouseIdList") Set<Long> warehouseIdList,
                           @Param("selfNo") String selfNo,
                           @Param("transportTypeList") Collection<String> transportTypeList
    );

    default IPage<BoxDO> selectSettlementPage(PageVO page, BoxSettlementQueryVO vo) {
//        LambdaQueryWrapper<BoxDO> boxQuery = new LambdaQuery<BoxDO>()
//                .likeIfPresent(BoxDO::getSelfNo, vo.getSelfNo())
//                .inIfPresent(BoxDO::getStartWarehouseId, vo.getStartWarehouseIdList())
//                .eqIfPresent(BoxDO::getShipmentStatus, vo.getBoxStatus())
//                .inIfPresent(BoxDO::getDestWarehouseId, vo.getDestWarehouseIdList())
//                .eqIfPresent(BoxDO::getSlStatus, vo.getSlStatus())
//                .orderByDesc(BoxDO::getId);

        String beginJSDate = null;
        String endJsDate = null;
        if (vo.getBeginJsDate() != null) {
            beginJSDate = DateUtil.format(vo.getBeginJsDate(), "yyyy-MM-dd HH:mm:ss");
            endJsDate = DateUtil.format(vo.getEndJsDate(), "yyyy-MM-dd HH:mm:ss");
        }
        String beginDaogangTime =null ;
        String endDaogangTime =null ;
        if(vo.getBeginDaogangTime()!=null ){
            beginDaogangTime = DateUtil.format(vo.getBeginDaogangTime(), "yyyy-MM-dd HH:mm:ss");
            endDaogangTime = DateUtil.format(vo.getEndDaogangTime(), "yyyy-MM-dd HH:mm:ss");
        }
        String beginQingguanTime = null ;
        String endQingguanTime =null ;
        if(vo.getBeginQingguanTime()!=null ){
            beginQingguanTime = DateUtil.format(vo.getBeginQingguanTime(), "yyyy-MM-dd HH:mm:ss");
            endQingguanTime = DateUtil.format(vo.getEndQingguanTime(), "yyyy-MM-dd HH:mm:ss");
        }

        String beginUlWarehouseTime = null ;
        String endUlWarehouseTime =null ;
        if(vo.getBeginQingguanTime()!=null ){
            beginUlWarehouseTime = DateUtil.format(vo.getBeginUlWarehouseTime(), "yyyy-MM-dd HH:mm:ss");
            endUlWarehouseTime = DateUtil.format(vo.getEndUlWarehouseTime(), "yyyy-MM-dd HH:mm:ss");
        }

        IPage<BoxDO> mpPage = MyBatisUtils.buildPage(page);
        return boxSettlePage(mpPage, vo.getStartWarehouseIdList(), vo.getDestWarehouseIdList(),
                vo.getSelfNo(), vo.getBoxStatus(), vo.getSlStatus(),
                beginJSDate, endJsDate,beginDaogangTime,endDaogangTime,beginQingguanTime,endQingguanTime,beginUlWarehouseTime,endUlWarehouseTime);
//        return selectPage(page, boxQuery);
    }

    @ResultType(BoxDO.class)
    @Select({
            "<script>",
            "SELECT",
            "box.*",
            "FROM",
            "ecw_box box",
            "<when test = 'beginJsDate != null or endJsDate != null'>",
            ", ecw_box_settlement settlement ",
            "</when>",

            "<when test = 'beginUlWarehouseTime != null or endUlWarehouseTime != null'>",
            ", ecw_box_clearance clearance ",
            "</when>",

            "WHERE",
            "box.deleted = 0 and box.pr_status &gt; 21",

            "<when test = 'beginJsDate != null or endJsDate != null'>",
            "and box.id = settlement.shipment_id and settlement.deleted = 0",
            "</when>",

            "<when test = 'beginUlWarehouseTime != null or endUlWarehouseTime != null'>",
            "and box.id = clearance.shipment_id and clearance.deleted = 0",
            "</when>",

            "<when test = 'beginJsDate != null'>",
            "and settlement.sl_settled_time &gt;= #{beginJsDate}",
            "</when>",

            "<when test = 'endJsDate != null'>",
            "and settlement.sl_settled_time &lt;= #{endJsDate}",
            "</when>",

            "<when test = 'beginDaogangTime != null'>",
            "and box.dg_date &gt;= #{beginDaogangTime}",
            "</when>",

            "<when test = 'endDaogangTime != null'>",
            "and box.dg_date &lt;= #{endDaogangTime}",
            "</when>",


            "<when test = 'beginQingguanTime != null'>",
            "and box.qg_date &gt;= #{beginQingguanTime}",
            "</when>",

            "<when test = 'endQingguanTime != null'>",
            "and box.qg_date &lt;= #{endQingguanTime}",
            "</when>",

            "<when test = 'beginUlWarehouseTime != null'>",
            "and clearance.cl_clear_time &gt;= #{beginUlWarehouseTime}",
            "</when>",

            "<when test = 'endUlWarehouseTime != null'>",
            "and clearance.cl_clear_time &lt;= #{endUlWarehouseTime}",
            "</when>",

            "<when test = 'selfNo != null and selfNo != \"\" '>",
            "and box.self_no LIKE concat('%',#{selfNo},'%')",
            "</when>",

            "<when test = 'startWarehouseIdList != null and startWarehouseIdList.size() != 0 '>",
            "AND (box.start_warehouse_id in ",
            "<foreach item='id' index='index' collection='startWarehouseIdList' open='(' separator=',' close=')'>",
            "#{id}",
            "</foreach>)",
            "</when>",

            "<when test = 'destWarehouseIdList != null and destWarehouseIdList.size() != 0 '>",
            "AND (box.dest_warehouse_id in ",
            "<foreach item='id' index='index' collection='destWarehouseIdList' open='(' separator=',' close=')'>",
            "#{id}",
            "</foreach>)",
            "</when>",

            "<when test = 'boxStatus != null'>",
            "and box.shipment_status = #{boxStatus}",
            "</when>",

            "<when test = 'slStatus != null'>",
            "and box.sl_status = #{slStatus}",
            "</when>",

            "</script>"
    })
    IPage<BoxDO> boxSettlePage(@Param("mpPage") IPage<BoxDO> page,
                               @Param("startWarehouseIdList") Collection<Long> startWarehouseIdList,
                               @Param("destWarehouseIdList") Collection<Long> destWarehouseIdList,
                               @Param("selfNo") String selfNo,
                               @Param("boxStatus") Integer boxStatus,
                               @Param("slStatus") Integer slStatus,
                               @Param("beginJsDate") String beginJsDate,
                               @Param("endJsDate") String endJsDate,

                               @Param("beginDaogangTime") String beginDaogangTime,
                               @Param("endDaogangTime") String endDaogangTime,
                               @Param("beginQingguanTime") String beginQingguanTime,
                               @Param("endQingguanTime") String endQingguanTime,
                               @Param("beginUlWarehouseTime") String beginUlWarehouseTime,
                               @Param("endUlWarehouseTime") String endUlWarehouseTime
                               );

    @Select("SELECT box.* from ecw_box box left join ecw_box_agent agent on box.id = agent.shipment_id\n" +
            "where box.pr_status = 25 and agent.agent_id is null and box.deleted=0")
    List<BoxDO> agentNoticeList();

    @Select("SELECT box.* from ecw_box box left join ecw_box_agent agent on box.id = agent.shipment_id\n" +
            "where box.checkout_status = 222 and agent.agent_id is null and box.deleted=0")
    List<BoxDO> agentAirNoticeList();

    @Select("SELECT box.* from ecw_box box left join ecw_box_customs customs on box.id = customs.shipment_id\n" +
            "where box.checkout_status = 222 and box.over_machine_status = 2 and box.over_machine_abnormal_status = 2 and customs.id is not null and box.deleted=0")
    List<BoxDO> overMachineNoticeList();

    @Select("select * from (select ${ew.sqlSelect} from ecw_box as box where box.deleted=0 and ${ew.sqlSegment}) as t")
    IPage<Map<String, Object>> boxEntityList(@Param(Constants.WRAPPER) Wrapper<BoxDO> ew, @Param("page") IPage page);


    @Select("SELECT" +
            " * " +
            " FROM" +
            "  ( " +
            "     SELECT " +
            "       box.*," +
            "       ifnull(preload.orderCount,0) orderCount," +
            "       ifnull(lading.ladingCount,0) ladingCount " +
            "     FROM " +
            "     ecw_box box " +
            "     LEFT JOIN ( SELECT shipment_id, COUNT(DISTINCT order_id) AS orderCount FROM ecw_box_preload_goods WHERE deleted = 0 AND is_remove = 0 GROUP BY shipment_id ) preload ON box.id = preload.shipment_id " +
            "     LEFT JOIN ( SELECT shipment_id, COUNT(*) AS ladingCount FROM ecw_make_bill_of_lading WHERE deleted = 0 AND STATUS = 2 GROUP BY shipment_id ) lading ON box.id = lading.shipment_id where box.deleted=0 " +
            "   ) q  ${ew.customSqlSegment}")
    IPage<Map<String, Object>> boxEntityList1(@Param(Constants.WRAPPER) Wrapper<BoxDO> ew, @Param("page") IPage page);

    List<MissNumberDO> getMissNumber(@Param("startWarehouseId") Long startWarehouseId,
                                     @Param("startCountryId") Long startCountryId,
                                     @Param("destCountryId") Long destCountryId,
                                     @Param("minNumber") Integer minNumber);

    Long getMinNumDeleteBox(@Param("startWarehouseId") Long startWarehouseId,
                            @Param("startCountryId") Long startCountryId,
                            @Param("destCountryId") Long destCountryId
    );


    @Override
    default PageResult<BoxDO> selectPage(PageVO page, Object object) {
        if (object instanceof BoxQueryVO) {
            BoxQueryVO vo = (BoxQueryVO) object;
            if (StringUtils.isEmpty(vo.getTransportType())) {
                vo.setTransportType(null);
            }
            List<Long> warehouseIdList = vo.getWarehouseIdList();
            return selectPage(page, new LambdaQuery<BoxDO>()

                    .eqIfPresent(BoxDO::getId, vo.getId())
                    .likeIfPresent(BoxDO::getSelfNo, vo.getSelfNo())
                    .likeIfPresent(BoxDO::getCubNo, vo.getCubNo())
                    .eqIfPresent(BoxDO::getCabinetId, vo.getCabinetId())
                    .eqIfPresent(BoxDO::getStartWarehouseId, vo.getStartWarehouseId())
                    .eqIfPresent(BoxDO::getStatus, vo.getStatus())
                    .eqIfPresent(BoxDO::getShipmentStatus, vo.getBoxStatus())
                    .eqIfPresent(BoxDO::getDestWarehouseId, vo.getDestWarehouseId())
                    .eqIfPresent(BoxDO::getTransportType, vo.getTransportType())
                    .inIfPresent(BoxDO::getTransportType, vo.getTransportTypeList())
                    .eqIfPresent(BoxDO::getShippingChannelId, vo.getShippingChannelId())
                    .betweenIfPresent(BoxDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .and(CollectionUtil.isNotEmpty(warehouseIdList),
                            i -> i.in(BoxDO::getStartWarehouseId, warehouseIdList)
                                    .or()
                                    .in(BoxDO::getDestWarehouseId, warehouseIdList)
                    )
                    .and(NumberUtils.hasCon(vo), wrapper -> {
                                NumberUtils.buildWrapper(BoxDO::getYzDate, vo.getBeginYzDate(), vo.getEndYzDate(), wrapper);
                                wrapper.or(NumberUtils.hasCon(vo.getBeginZgDate(), vo.getEndZgDate()), zgWrapper -> NumberUtils.buildWrapper(BoxDO::getZgDate, vo.getBeginZgDate(), vo.getEndZgDate(), zgWrapper));
                                wrapper.or(NumberUtils.hasCon(vo.getBeginQyDate(), vo.getEndQyDate()), qyWrapper -> NumberUtils.buildWrapper(BoxDO::getQyDate, vo.getBeginQyDate(), vo.getEndQyDate(), qyWrapper));
                                wrapper.or(NumberUtils.hasCon(vo.getBeginDgDate(), vo.getEndDgDate()), dgWrapper -> NumberUtils.buildWrapper(BoxDO::getDgDate, vo.getBeginDgDate(), vo.getEndDgDate(), dgWrapper));
                                wrapper.or(NumberUtils.hasCon(vo.getBeginQgDate(), vo.getEndQgDate()), qgWrapper -> NumberUtils.buildWrapper(BoxDO::getQgDate, vo.getBeginQgDate(), vo.getEndQgDate(), qgWrapper));
                                wrapper.or(NumberUtils.hasCon(vo.getBeginThDate(), vo.getEndThDate()), xgWrapper -> NumberUtils.buildWrapper(BoxDO::getThDate, vo.getBeginThDate(), vo.getEndThDate(), xgWrapper));
                            }
                    )
                    .orderByDesc(BoxDO::getId));
        }

        return null;
    }

    @ResultType(BoxDO.class)
    @Select({
            "<script>",
            "SELECT",
            "box.*",
            "FROM",
            "ecw_box box",
            "WHERE",
            "box.deleted = 0",
            "and box.pr_status = 25",
            "and box.ty_status = 2111",
            "AND box.transport_type in ",
            "<foreach item='id' index='index' collection='transportTypeList' open='(' separator=',' close=')'>",
            "#{id}",
            "</foreach>",
            "AND (box.start_warehouse_id in ",
            "<foreach item='id' index='index' collection='warehouseIdList' open='(' separator=',' close=')'>",
            "#{id}",
            "</foreach> or box.dest_warehouse_id in ",
            "<foreach item='id' index='index' collection='warehouseIdList' open='(' separator=',' close=')'>",
            "#{id}",
            "</foreach>)",
            "<when test = 'selfNo != null and selfNo != \"\" '>",
            "and box.self_no LIKE concat('%',#{selfNo},'%')",
            "</when>",
            "</script>"
    })
    IPage<BoxDO> tallyAirPage(@Param("mpPage") IPage<BoxDO> page,
                              @Param("warehouseIdList") Set<Long> warehouseIdList,
                              @Param("selfNo") String selfNo,
                              @Param("transportTypeList") Collection<String> transportTypeList
    );
}
