package cn.iocoder.yudao.module.shipment.service.box;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.order.dal.dataobject.approval.OrderApprovalDO;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderConsignee.OrderConsigneeDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderItem.OrderItemDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderSplitItem.OrderSplitItemDO;
import cn.iocoder.yudao.module.order.enums.OrderAirTimeEnum;
import cn.iocoder.yudao.module.order.enums.OrderSeaTimeEnum;
import cn.iocoder.yudao.module.order.enums.TargetLogEnum;
import cn.iocoder.yudao.module.order.vo.order.OrderBackPageVO;
import cn.iocoder.yudao.module.order.vo.orderSplit.OrderSplitApplyDetailDto;
import cn.iocoder.yudao.module.order.vo.orderSplit.OrderSplitBackVO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductPriceDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.*;
import cn.iocoder.yudao.module.shipment.enums.BoxAirStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.BoxStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.OrderShipmentLog;
import cn.iocoder.yudao.module.shipment.vo.box.*;
import cn.iocoder.yudao.module.shipment.vo.boxCabinet.BoxCabinetApplyReq;
import cn.iocoder.yudao.module.shipment.vo.boxCabinet.BoxCabinetLoadVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinetUnload.BoxUnloadApplyReq;
import cn.iocoder.yudao.module.shipment.vo.boxCustoms.BoxCustomsApplyReq;
import cn.iocoder.yudao.module.shipment.vo.boxLoadInfo.BoxLoadBatchRemoveReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxLoadInfo.BoxLoadInfoBatchAddReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxOrderExit.BoxOrderExitCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxLoadOrderVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxPreloadGoodsBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadSection.BoxLoadDetailBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadSection.BoxLoadSectionBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxTally.BoxOrderLocationUpdateReq;
import cn.iocoder.yudao.module.shipment.vo.boxTally.BoxTallyBackVO;
import cn.iocoder.yudao.module.system.enums.sms.SmsSceneEnum;
import org.springframework.context.annotation.Description;

import javax.validation.Valid;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * 出货 Service 接口
 *
 * @author lalala
 */
public interface BoxService extends IService<BoxDO> {

    /**
     * 创建出货
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createBox(@Valid BoxCreateReqVO createReqVO);

    /**
     * 更新出货
     *
     * @param updateReqVO 更新信息
     */
    void updateBox(@Valid BoxUpdateReqVO updateReqVO);

    /**
     * 删除出货
     *
     * @param id 编号
     */
    void deleteBox(Long id);

    /**
     * 获得出货
     *
     * @param id 编号
     * @return 出货
     */
    BoxBackVO getBoxDetail(Long id);

    public Integer getPlanNum(Long shipentId);

    BoxDO getBox(Long id);

    /**
     * 获得出货列表
     *
     * @param ids 编号
     * @return 出货列表
     */
    List<BoxDO> getBoxList(Collection<Long> ids);

    /**
     * 获得出货分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 出货分页
     */
    PageResult<BoxBackVO> getBoxPage(BoxQueryVO query, PageVO page);

    /**
     * 获得出货列表, 用于 Excel 导出
     *
     * @param query 查询
     * @return 出货列表
     */
    List<BoxDO> getBoxList(BoxQueryVO query);

    /**
     * 设置代理商
     *
     * @param shipmentId 出货单ID
     * @param agentId    代理商ID
     */
    void updateAgent(Long shipmentId, Long agentId);

    /**
     * 配船设置
     *
     * @param shipmentId   出货单ID
     * @param saExmtStatus 状态
     * @param configTime   配船时间
     * @param operateType  操作类型
     */
    void shipConfigure(Long shipmentId, Integer saExmtStatus, Integer operateType, String configTime);

    /**
     * 获取出货单基本信息
     *
     * @param shipmentId shipmentId
     * @return
     */
    BoxBaseBackVO getBaseBoxInfo(Long shipmentId);

    /**
     * 发送短信
     *
     * @param shipment     出货单
     * @param smsSceneEnum 场景
     */
    void sendSms(Long shipment, SmsSceneEnum smsSceneEnum);

    void sendSms(Long shipmentId, Collection<Long> orderIdList, SmsSceneEnum smsSceneEnum);

    /**
     * 新增销售日志
     *
     * @param shipmentId 出货单ID
     */
    void addTargetLog(Long shipmentId, TargetLogEnum targetLogEnum, Date date);

    /**
     * 更新出货单下的订单状态
     *
     * @param shipmentId       出货单ID
     * @param orderStatus      订单状态
     * @param inWarehouseState 入仓状态
     * @param shipmentState    出货单状态
     * @param auditType        审核类型
     * @param auditResult      审核结果
     */
    @Deprecated
    @Description(value = "当前方法已废弃")
    void updateOrderStatusByShipmentId(Long shipmentId, Integer orderStatus, Integer inWarehouseState, Integer shipmentState, Integer auditType, String auditResult);

    void updateOrderStatusByShipmentIdAndTransportType(Long shipmentId, Integer orderStatus, Integer inWarehouseState, Integer shipmentState, Integer auditType, String auditResult, Integer transportType);

    void updateOrderStatusByShipmentIdAndTransportType(Long shipmentId, Integer orderStatus, Integer inWarehouseState, Integer shipmentState, Integer auditType, String auditResult, Date businessTime, Integer transportType);

    void updateOrderStatusByShipmentIdAndTransportType(Long shipmentId, Integer orderStatus, Integer inWarehouseState, Integer shipmentState, Integer auditType, String auditResult, Date businessTime, Date estTime, Integer transportType);

    void updateOrderStatusByOrderIdAndTransportType(Collection<Long> orderIdList, Integer orderStatus, Integer inWarehouseState, Integer shipmentState, Integer auditType, String auditResult, Date businessTime, Date estTime, Integer transportType);

    @Deprecated
    @Description(value = "当前方法已废弃")
    void updateOrderStatusByShipmentId(Long shipmentId, Integer orderStatus, Integer inWarehouseState, Integer shipmentState, Integer auditType, String auditResult, Date date);

    void updateOrderStatus(Long orderId, Integer orderStatus, Integer inWarehouseState, Integer shipmentState, Integer auditType, String auditResult, Boolean is, Integer hasExit);

    void updateOrderStatus(Long orderId, Integer orderStatus, Integer inWarehouseState, Integer shipmentState, Integer auditType, String auditResult, Boolean is);

    /**
     * 添加空运订单动态日志
     *
     * @param orderIdList      订单id列表
     * @param orderAirTimeEnum {@link OrderAirTimeEnum} 空运动态枚举值
     * @param reasonZh         业务说明
     * @param reasonEn         业务说明
     * @param estTime          预计时间
     * @param businessTime     业务时间
     * @param now              当前执行时间
     */
    void addOrderAirTimeLog(Collection<Long> orderIdList, OrderAirTimeEnum orderAirTimeEnum, String reasonZh, String reasonEn, Date estTime, Date businessTime, Date now);

    void addOrderSeaTimeLog(Collection<Long> orderIdList, OrderSeaTimeEnum orderSeaTimeEnum, String reasonZh, String reasonEn, Date estTime, Date businessTime, Date now);

    /**
     * 校验已装箱数量是否符合
     *
     * @param shipmentId 出货单ID
     */
    void checkInstallNum(Long shipmentId);

    /**
     * 校验箱数量是否符合
     *
     * @param id 理货时传出货单ID，合包封箱时传合包id
     * @param type 0-理货 1-合包
     */
    void checkOrderTagNum(Long id,Integer type);

    /**
     * 校验已卸柜数量是否符合
     *
     * @param shipmentId 出货单ID
     */
    void checkUnLoadNum(Long shipmentId);

    BoxLoadDetailBackVO boxGoodsList(Long shipmentId, Long secId, Integer isCover, Integer isRemove);


    /**
     * app端获取我的装柜
     *
     * @param query 条件
     * @return
     */
    PageResult<BoxBackVO> getBoxLoadPage(MyBoxLoadQueryVO query);

    /**
     * 获得待理货列表
     *
     * @param query 条件
     * @return
     */
    PageResult<BoxBackVO> getBoxTallyPage(MyBoxLoadQueryVO query);

    /**
     * app端获取我的卸柜
     *
     * @param query 条件
     * @return
     */
    PageResult<BoxBackVO> getBoxUnloadPage(MyBoxUnloadQueryVO query);

    /**
     * @param shipmentId 出货单ID
     * @return
     */
    List<BoxTallyBackVO> tallyList(Long shipmentId);


    /**
     * 批量修改储位
     *
     * @param req 修改参数
     */
    void batchOrderLocationUpdate(BoxOrderLocationUpdateReq req);

    /**
     * 理货提交
     *
     * @param shipmentId 提货单ID
     */
    void tallyCommit(Long shipmentId);

    /**
     * 获取装柜详情
     *
     * @param shipmentId ID
     * @return
     */
    BoxCabinetLoadVO getBoxCabinetDetail(Long shipmentId);

    /**
     * 获取卸柜详情
     *
     * @param shipmentId ID
     * @return
     */
    BoxCabinetLoadVO getBoxCabinetUnloadDetail(Long shipmentId);

    /**
     * 封柜申请
     *
     * @param applyReq 提交信息
     */
    void closureApply(BoxCabinetApplyReq applyReq);

    /**
     * pda 批量装柜
     *
     * @param createReqVO 参数
     */
    void boxLoadBatchCreate(BoxLoadInfoBatchAddReqVO createReqVO);

    /**
     * pwa 批量纠错
     *
     * @param createReqVO 参数
     */
    void batchRemove(BoxLoadBatchRemoveReqVO createReqVO);

    /**
     * 卸柜申请
     *
     * @param applyReq 参数
     */
    void unloadApply(BoxUnloadApplyReq applyReq);

    /**
     * 退场出货单
     *
     * @param query 参数
     * @return
     */
    PageResult<BoxBackVO> getBoxExitPage(MyBoxExitQueryVO query);

    /**
     * 退场订单
     *
     * @param createReqVO 参数
     */
    void exitOrder(BoxOrderExitCreateReqVO createReqVO);

    /**
     * 获得退场详情
     *
     * @param shipmentId 出货单ID
     * @return
     */
    BoxCabinetLoadVO getBoxCabinetExitDetail(Long shipmentId);

    /**
     * 退场订单列表
     *
     * @param shipmentId 出货单ID
     * @return
     */
    List<BoxLoadOrderVO> getBoxCabinetExitList(Long shipmentId);

    /**
     * 确认到仓
     *
     * @param applyReq 请求参数
     */
    void confirmArrival(BoxCustomsApplyReq applyReq);

    /**
     * 更换货柜号重发
     *
     * @param req 请求参数
     */
    void changeContainer(BoxChangeReqVO req);

    List<OrderSplitBackVO> listByOrderId(Long orderId);


    List<OrderSplitItemDO> splitItemListByOrderId(Long orderItemId);

    List<OrderItemDO> getOrderItemsByOrderId(Long orderId);

    /**
     * 修改订单状态
     *
     * @param orderId          订单ID 与订单号选择一个填写
     * @param orderNo          订单号 与订单ID选择一个填写
     * @param status           订单状态 订单主状态
     * @param abnormalState    订单异常状态 0 原始状态
     * @param inWarehouseState 入仓状态 0 原始状态
     * @param shipmentState    出货状态 0 原始状态
     * @param auditType        审核类型 0 原始状态 方便前端根据类型跳转相关的审批页面去处理相应的业务
     * @param auditResult      审批结果(尽量精简的备注说明)
     */
    void updateStatus(Long orderId, String orderNo, Integer status, Integer abnormalState,
                      Integer inWarehouseState, Integer shipmentState, Integer auditType, String auditResult);

    /**
     * 拆单取消
     *
     * @param orderId 订单ID
     */
    void splitCancelByOrderId(Long orderId);

    /**
     * 拆单成功
     *
     * @param orderSplitApplyDetailDto
     * @param auditResult
     */
    void splitSuccess(OrderSplitApplyDetailDto orderSplitApplyDetailDto, String auditResult);

    OrderDO getSplitBoxOrder(String orderNo);

    /**
     * 修改
     *
     * @param shipmentId 出货ID
     * @param oldOrder   旧订单信息
     * @param newOrderId 拆单后的订单ID
     * @param newOrderNo 拆单后的订单编号
     */
    void updateSplitOrderId(Long shipmentId, OrderDO oldOrder, Long newOrderId, String newOrderNo);


    /**
     * 修改
     *
     * @param shipmentId     出货ID
     * @param oldOrder       旧订单信息
     * @param exitSplitOrder 部分退场拆出的子订单
     */
    void updateExitSplitOrderId(Long shipmentId, OrderDO oldOrder, List<OrderDO> exitSplitOrder);

    /**
     * 弹出提醒列表
     *
     * @return
     */
    List<BoxNoticeVO> noticeList(BoxQueryVO boxQueryVO);

    /**
     * 生成提货单号
     *
     * @param shipmentId 出货单ID
     */
    void generateBillLadingNo(Long shipmentId, boolean updateOrder, boolean tempSort);

    BoxLoadOrderVO searchOrderByBoxNo(BoxOrderSearchVO boxOrderSearchVO);

    BoxLoadOrderVO searchLoadOrderByBoxNo(BoxOrderSearchVO boxOrderSearchVO);

    List<BoxExportVO> getExportBoxList(BoxQueryVO query);

    /**
     * 生成提货异常
     *
     * @param shipmentId 出货单
     */
    void createPickUpException(Long shipmentId);

    /**
     * 校验出货单是否在审核中
     *
     * @param shipmentId 出货单ID
     */
    void checkApproval(Long shipmentId, Integer approvalType);

    /**
     * 校验密码
     *
     * @param inputPwd
     */
    void checkPwd(String inputPwd);

    /**
     * 获取拆单列表
     *
     * @param shipmentId 出货单ID
     * @param orderId    订单ID
     */
    List<OrderSplitBackVO> splitList(Long shipmentId, Long orderId);

    void updateOrderSelfNoByShipmentId(Long shipmentId);

    OrderDO getOrder(Long orderId);

    /**
     * 获取订单收货人
     *
     * @param orderDO
     * @return
     */
    OrderConsigneeDO getOrderConsignee(OrderDO orderDO);

    /**
     * 检查是否有预装订单
     *
     * @param shipmentId 出货单ID
     */
    void checkPreInstallNum(Long shipmentId);

    /**
     * 创建代收货款异常
     *
     * @param shipmentId 出货单ID
     */
    void createPaymentException(Long shipmentId);

    /**
     * 根据订单找出订单项
     *
     * @param orderIdList 订单ID列表
     * @return
     */
    List<OrderItemDO> getOrderItemListByOrderIdList(List<Long> orderIdList);


    /**
     * 获取路线价格
     *
     * @param proId     商品ID
     * @param lineId    路线ID
     * @param channelId 渠道ID
     * @return
     */
    ProductPriceDO getProductPrice(Long proId, Long lineId, Long channelId);

    /**
     * 找出这些订单中审核中的订单
     *
     * @param oIdList 订单ID
     * @return
     */
    List<OrderApprovalDO> getOrderApprovalIng(Set<Long> oIdList);

    Boolean getMultiSpecification(List<BoxPreloadGoodsBackVO> goodsList);

    List<BoxLoadSectionBackVO> pdaTallyList(Long shipmentId);

    /**
     * 获取出货异常列表
     *
     * @param shipmentId 出货单ID
     * @return
     */
    List<BoxAbnormalDO> getAbnormalList(Long shipmentId);

    List<BoxAbnormalDO> getAbnormalList(boolean airShipment);

    /**
     * 更新异常订单
     *
     * @param dealReqVO 参数
     */
    void updateAbnormalOrder(OrderDealReqVO dealReqVO);

    void updateAbnormalOrderAir(OrderDealReqVO dealReqVO);

    /**
     * 根据出货单筛选报关类型的订单
     *
     * @param shipmentId   出货单ID
     * @param customsTypes 报关方式
     * @return
     */
    List<OrderBackPageVO> customsOrderList(Long shipmentId, String customsTypes);

    /**
     * 重置提单号
     *
     * @param orderId
     */
    void resetOrderTidanNo(Long orderId, boolean hasExit);

    void buildSpecialReceivable(Long shipmentId);

    void sendCabinetMessage(Long shipmentId);

    void shipmentInternalMessage(Long shipmentId, String selfNo, String date, BoxStatusEnum boxStatusEnum, Integer transportType);

    void shipmentAirInternalMessage(Long shipmentId, String selfNo, String date, BoxAirStatusEnum boxAirStatusEnum, Integer transportType);

    void shipmentAirOrderInternalMessage(Collection<Long> orderList, String selfNo, String date, BoxAirStatusEnum boxAirStatusEnum, Integer transportType);

    /**
     * 获取装柜信息
     *
     * @param shipmentId 出货单ID
     * @return
     */
    BoxCabinetDO getBoxCabinetDO(Long shipmentId);

    /**
     * 获取卸柜信息
     *
     * @param shipmentId 出货单ID
     * @return
     */
    BoxCabinetUnloadDO getBoxCabinetUnloadDO(Long shipmentId);

    /**
     * 清空订单表的自编号和提单号
     *
     * @param shipmentId       出货单ID
     * @param clearContainerNo 重置柜号
     */
    void clearOrderTidanNo(Long shipmentId, boolean clearContainerNo);

    void resetOrderTidanNo(Long orderId, boolean clearContainerNo, boolean hasExit);

    void updateOrderSelfNo(Long orderId, String selfNo);

    /**
     * 校验订单是否预装在出货单中
     *
     * @param orderId 订单ID
     * @return
     */
    BoxDO checkOrderScheduling(Long orderId);

    /**
     * 更新订单预计到港时间
     *
     * @param selfNo      自编号
     * @param transportId 运输方式
     * @param estTime     预计时间
     */
    void updateOrderArriveTime(String selfNo, Integer transportId, Date estTime);

    /**
     * 更新订单预计时间
     *
     * @param shipmentId 出货id
     * @param transportId 运输方式
     * @param sailTime 预计时间
     */
    void updateOrderSailTime(Long shipmentId, Integer transportId, Date sailTime);

    /**
     * 添加出货操作日志
     *
     * @param orderIdList      订单列表
     * @param orderShipmentLog 操作类型
     */
    void addOrderLog(Collection<Long> orderIdList, OrderShipmentLog orderShipmentLog, String message);

    /**
     * 删除拆单草稿信息
     *
     * @param orderId 订单ID
     */
    void deleteSplitItem(Long orderId);

    void addOrderLogByShipmentId(Long shipmentId, OrderShipmentLog cabinetUnload, String message);

    void addInstallCompleteLog(Long shipmentId);

    /**
     * 修改拖车货柜号
     *
     * @param boxDO 出货
     */
    void updateTrailerCubNo(BoxDO boxDO);

    /**
     * 清空提单信息
     *
     * @param orderIdList 订单
     */
    void removeLadingBill(Collection<Long> orderIdList);

    void clearLadingBill(Long shipmentId);

    /**
     * 获取出货未核销数量
     *
     * @param shipmentId 出货
     * @return
     */
    long getUnWriteOffCount(Long shipmentId);

    /**
     * 批量校验订单是否有在排单中
     *
     * @param orderIds 订单id列表
     * @return
     */
    List<Long> checkOrdersScheduling(List<Long> orderIds);

    List<OrderSplitBackVO> partRefundSplitList(Long shipmentId, Long orderId);


    /**
     * 创建报关拆单通知
     *
     * @param shipmentId  出货ID
     * @param orderIdList 拆单的订单列表
     * @param type        类型
     */
    void createCustomsSplitNotify(Long shipmentId, List<Long> orderIdList, Integer type);


    void dealCustomsSplitNotify(Long notifyId);

    void splitConsistencyCheck(List<OrderSplitItemDO> orderSplitItemDOList, Integer allNum, Integer quantity,
                               BigDecimal weight, BigDecimal volume, BigDecimal chargeWeight,
                               BigDecimal chargeVolume, BigDecimal chargeQuantity);


    void splitWorthConsistencyCheck(List<OrderSplitItemDO> orderAllSplitItemDOList, BigDecimal worth);

    void checkOrdersApprovaling(Long shipmentId);

    void splitUsed(Long spiltId);

    List<BoxNoticeVO> airNoticeList(BoxQueryVO boxQueryVO);

    List<String> getAttrNameByIdList(String attrIds);

    void updateExitOrderRefundNum(Long id, Integer refundNum);

    /**
     * 批量加价审核成功
     *
     * @param shipmentId 出货单
     * @param details    加价详情
     */
    void batchAddPriceSuccess(Long shipmentId, String details);

    void asyncRemoveOrder(Collection<Long> orderIdCollection, Integer type, boolean hasExit, String transportType);

    PageResult<BoxFinanceBackVO> getBoxSettlementPage(BoxSettlementQueryVO query, PageVO page);

    void sendPreloadMessage(Long shipmentId);

    BoxSettlementExcelVO getShipmentCostSummaryDto(BoxFinanceBackVO back);

    /**
     * 空运装柜
     *
     * @param shipmentId 出货单
     */
    void airLoadInfo(Long shipmentId);

    PageResult<BoxBackVO> getBoxTallyPageAir(MyBoxLoadQueryVO query);

    PageResult<BoxBackVO> getBoxExitPageAir(MyBoxExitQueryVO query);

    void updateOrderStatusByOrderList(Collection<Long> orderIdList, Integer orderStatus, Integer inWarehouseState, Integer shipmentState, Integer auditType, String auditResult, Date estTime, Date businessTime);

    /**
     * 删除已清关的订单
     *
     * @param shipmentId
     * @param backOrderId
     */
    void deleteClearanceOrder(Long shipmentId, Long backOrderId);

    /**
     * 校验订单是否已清关
     *
     * @param shipmentId
     * @param orderId
     */
    void checkIsInClearance(Long shipmentId, Long orderId);

    void deleteArrivalOrderByShipment(Long shipmentId);

    void deleteClearanceOrderByShipment(Long shipmentId);

    /**
     * 批量加价校验订单是否已经预付
     *
     * @param orderIdList 订单列表
     */
    void checkOrderPayStatus(BoxOrderMarkUpVO vo);

    void updateUlBoxTime(Long shipmentId, Date ulBoxTime);

    /**
     * 获取最近的审核成功记录
     *
     * @param shipmentId
     * @return
     */
    BoxApprovalDO getLatestApproval(Long shipmentId);

    /**
     * 订单日志回退
     *
     * @param transportType 运输方式
     * @param selfNo        自编号
     */
    void backOrderLog(String transportType, String selfNo);

    void backOrderLog(String transportType, Collection<Long> orderIds);

    /**
     * 锁定价格
     * @param shipmentId
     */
    void lockOrderPrice(Long shipmentId);

    /**
     * 解锁价格
     * @param shipmentId
     */
    void unlockOrderPrice(Long shipmentId);

    Integer getRealNum(Long shipmentId);

    void updateTallyStatus(Long shipmentId, Collection<Long> orderIdList);

    /**
     * 清空卸柜箱数
     * @param shipmentId  出货单ID
     */
    void clearUnloadCabinet(Long shipmentId);
}