package cn.iocoder.yudao.module.shipment.vo.box;

import lombok.*;
import java.util.*;
import io.swagger.annotations.*;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import org.springframework.format.annotation.DateTimeFormat;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

@Data
@ApiModel("管理后台 - 出货查询 VO")
public class BoxQueryVO {

    @ApiModelProperty(value = "主键ID")
    private Long id;

    @ApiModelProperty(value = "自编号")
    private String selfNo;

    @ApiModelProperty(value = "柜号")
    private String cubNo;

    @ApiModelProperty(value = "柜型ID")
    private Long cabinetId;

    @ApiModelProperty(value = "始发地id")
    private Long startWarehouseId;

    @ApiModelProperty(value = "发货状态:见字典")
    private Integer status;

    @ApiModelProperty(value = "目的地id")
    private Long destWarehouseId;

    @ApiModelProperty(value = "运输方式")
    private String transportType;

    @ApiModelProperty(value = "运输方式列表")
    private List<String> transportTypeList;

    @ApiModelProperty(value = "出货渠道ID")
    private Long shippingChannelId;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始预装时间")
    private Date beginYzDate;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束预装时间")
    private Date endYzDate;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始装柜时间")
    private Date beginZgDate;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束装柜时间")
    private Date endZgDate;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始起运时间")
    private Date beginQyDate;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束起运时间")
    private Date endQyDate;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始到港时间")
    private Date beginDgDate;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束到港时间")
    private Date endDgDate;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始清关时间")
    private Date beginQgDate;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束清关时间")
    private Date endQgDate;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始提货时间")
    private Date beginThDate;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束提货时间")
    private Date endThDate;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始出仓时间")
    private Date beginCkDate;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束出仓时间")
    private Date endCkDate;

    @ApiModelProperty(value = "出货状态")
    private Integer boxStatus;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始创建时间")
    private Date beginCreateTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束创建时间")
    private Date endCreateTime;

    @ApiModelProperty(value = "国家ID")
    private Long countryId;
    //选择国家后，国家的仓库id列表（始发或者目的）
    private List<Long> warehouseIdList;

    //日期类型
    private Integer dateType;
    //时间区间
    private String[] dateRangeCreateTime;

    @ApiModelProperty(value = "提单状态, 0-未完成，1-部分完成，2-已完成")
    private Integer ladingBillStatus;

    @ApiModelProperty(value = "空运出货状态")
    private Integer shipmentStatusAir;

    @ApiModelProperty(value = "目的港清关：1、我司清关 2、合作方清关 3、客户")
    private Integer destinationClearance;

    @ApiModelProperty(value = "发货方式：1、多票，2、单票")
    private Integer deliveryType;

    @ApiModelProperty(value = "始发仓多选")
    private List<Long> startWarehouseIdList;

    @ApiModelProperty(value = "目的国多选")
    private List<Long> countryIdList;

    @ApiModelProperty(value = "目的城市多选")
    private List<Long> destCityIdList;

    @ApiModelProperty(value = "目的仓多选")
    private List<Long> destWarehouseIdList;

    @ApiModelProperty(value = "状态多选")
    private List<Integer> boxStatusList;

    @ApiModelProperty(value = "起运港多选")
    private List<Long> bigPortIdList;

    @ApiModelProperty(value = "目的港多选")
    private List<Long> destPortIdList;

    @ApiModelProperty(value = "订舱公司多选")
    private List<Long> spaceCompanyIdList;

    @ApiModelProperty(value = "驳船港多选")
    private List<Long> bargePortIdList;

    /**
     * 预计驳船时间
     */
    @ApiModelProperty(value = "预计驳船开始时间")
//    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String bargeBeginTime;
    @ApiModelProperty(value = "预计驳船结束时间")
//    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String bargeEndTime;
    /**
     * 预计开船时间
     */
    @ApiModelProperty(value = "预计开船开始时间")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String sailBeginTime;
    @ApiModelProperty(value = "预计开船结束时间")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String sailEndTime;

    /**
     * 截补料时间
     */
    @ApiModelProperty(value = "截补料开始时间")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String ladingBillBeginTime;
    @ApiModelProperty(value = "截补料结束时间")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String ladingBillEndTime;

    /**
     * VGM提交时间
     */
    @ApiModelProperty(value = "VGM提交开始时间")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String customsBeginTime;
    @ApiModelProperty(value = "VGM提交结束时间")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String customsEndTime;

    /**
     * 截关时间
     */
    @ApiModelProperty(value = "截关开始时间")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String cutOffBeginTime;
    @ApiModelProperty(value = "截关结束时间")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String cutOffEndTime;

    @ApiModelProperty(value = "报关方式多选-海运空运共用")
    private List<Integer> dcCustomsTypeList;
    @ApiModelProperty(value = "报关公司多选-海运空运共用")
    private List<Integer> dcCompanyIdList;
    @ApiModelProperty(value = "放行开始时间-海运空运共用")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String dcPassBeginTime;
    @ApiModelProperty(value = "放行结束时间-海运空运共用")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String dcPassEndTime;

    /**
     * 实际开船时间
     */
    @ApiModelProperty(value = "实际开船开始时间")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String dtRealShipBeginTime;
    @ApiModelProperty(value = "实际开船结束时间")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String dtRealShipEndTime;
    /**
     * 预计到港时间
     */
    @ApiModelProperty(value = "预计到港开始时间")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String dtEstArrivalBeginTime;
    @ApiModelProperty(value = "预计到港结束时间")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String dtEstArrivalEndTime;

    @ApiModelProperty(value = "出单方式多选")
    private List<Integer> cdOutBillTypeList;
    @ApiModelProperty(value = "Agent list状态")
    private Integer cdAgentlistType;
    @ApiModelProperty(value = "Soncap状态")
    private Integer cdSoncapType;

    /**
     * 预计清关时间
     */
    @ApiModelProperty(value = "预计清关开始时间")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String clEstBeginTime;

    /**
     * 预计清关时间
     */
    @ApiModelProperty(value = "预计清关结束时间")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String clEstEndTime;

    //---------------------------空运--------------------------

    @ApiModelProperty(value = "目的港清关多选")
    private List<Integer> destinationClearanceList;

    @ApiModelProperty(value = "发货方式多选")
    private List<Integer> deliveryTypeList;

    @ApiModelProperty(value = "出货渠道多选")
    private List<Long> shippingChannelIdList;

    @ApiModelProperty(value = "空运起运港多选")
    private List<Long> departurePortIdList;

    @ApiModelProperty(value = "空运目的港多选")
    private List<Long> airDestPortIdList;

    @ApiModelProperty(value = "航程")
    private Integer voyage;

    @ApiModelProperty(value = "订舱代理多选")
    private List<Long> shipperIdList;

    @ApiModelProperty(value = "航空公司多选")
    private List<Long> airlineCompanyIdList;

    @ApiModelProperty(value = "预计起飞开始时间")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String flyBeginTime;

    @ApiModelProperty(value = "预计起飞结束时间")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String flyEndTime;

    @ApiModelProperty(value = "代理公司Agent多选")
    private List<Long> agentIdList;

}
